@@ -1,227 +1,282 @@
 package Function::Parameters;
 
 use v5.14.0;
-
 use warnings;
 
 use Carp qw(confess);
 
 use XSLoader;
 BEGIN {
-	our $VERSION = '1.0103_01';
-	our $XS_VERSION = $VERSION;
-	$VERSION = eval $VERSION;
-	XSLoader::load;
+    our $VERSION = '1.0603_01';
+    our $XS_VERSION = $VERSION;
+    $VERSION = eval $VERSION;
+    XSLoader::load;
 }
 
 sub _assert_valid_identifier {
-	my ($name, $with_dollar) = @_;
-	my $bonus = $with_dollar ? '\$' : '';
-	$name =~ /^${bonus}[^\W\d]\w*\z/
-		or confess qq{"$name" doesn't look like a valid identifier};
+    my ($name, $with_dollar) = @_;
+    my $bonus = $with_dollar ? '\$' : '';
+    $name =~ /^${bonus}[^\W\d]\w*\z/
+        or confess qq{"$name" doesn't look like a valid identifier};
 }
 
 sub _assert_valid_attributes {
-	my ($attrs) = @_;
-	$attrs =~ /^\s*:\s*[^\W\d]\w*\s*(?:(?:\s|:\s*)[^\W\d]\w*\s*)*(?:\(|\z)/
-		or confess qq{"$attrs" doesn't look like valid attributes};
+    my ($attrs) = @_;
+    $attrs =~ m{
+        ^ \s*+
+        : \s*+
+        (?&ident) (?! [^\s:(] ) (?&param)?+ \s*+
+        (?:
+            (?: : \s*+ )?
+            (?&ident) (?! [^\s:(] ) (?&param)?+ \s*+
+        )*+
+        \z
+
+        (?(DEFINE)
+            (?<ident>
+                [^\W\d]
+                \w*+
+            )
+            (?<param>
+                \(
+                [^()\\]*+
+                (?:
+                    (?:
+                        \\ .
+                    |
+                        (?&param)
+                    )
+                    [^()\\]*+
+                )*+
+                \)
+            )
+        )
+    }sx or confess qq{"$attrs" doesn't look like valid attributes};
+}
+
+sub _reify_type_default {
+    require Moose::Util::TypeConstraints;
+    Moose::Util::TypeConstraints::find_or_create_isa_type_constraint($_[0])
+}
+
+sub _delete_default {
+    my ($href, $key, $default) = @_;
+    exists $href->{$key} ? delete $href->{$key} : $default
 }
 
 my @bare_arms = qw(function method);
 my %type_map = (
-	function => {
-		name => 'optional',
-		default_arguments => 1,
-		check_argument_count => 0,
-		named_parameters => 1,
-		types => 1,
-	},
-	method   => {
-		name => 'optional',
-		default_arguments => 1,
-		check_argument_count => 0,
-		named_parameters => 1,
-		types => 1,
-		attrs => ':method',
-		shift => '$self',
-		invocant => 1,
-	},
-	classmethod   => {
-		name => 'optional',
-		default_arguments => 1,
-		check_argument_count => 0,
-		named_parameters => 1,
-		types => 1,
-		attributes => ':method',
-		shift => '$class',
-		invocant => 1,
-	},
+    function           => {},  # all default settings
+    function_strict    => {
+        defaults   => 'function',
+        strict     => 1,
+    },
+    method             => {
+        defaults   => 'function',
+        attributes => ':method',
+        shift      => '$self',
+        invocant   => 1,
+    },
+    method_strict      => {
+        defaults   => 'method',
+        strict     => 1,
+    },
+    classmethod        => {
+        defaults   => 'method',
+        shift      => '$class',
+    },
+    classmethod_strict => {
+        defaults   => 'classmethod',
+        strict     => 1,
+    },
 );
-for my $k (keys %type_map) {
-	$type_map{$k . '_strict'} = {
-		%{$type_map{$k}},
-		check_argument_count => 1,
-	};
-}
+
+our @type_reifiers = \&_reify_type_default;
 
 sub import {
-	my $class = shift;
+    my $class = shift;
 
-	if (!@_) {
-		@_ = {
-			fun => 'function',
-			method => 'method',
-		};
-	}
-	if (@_ == 1 && $_[0] eq ':strict') {
-		@_ = {
-			fun => 'function_strict',
-			method => 'method_strict',
-		};
-	}
-	if (@_ == 1 && ref($_[0]) eq 'HASH') {
-		@_ = map [$_, $_[0]{$_}], keys %{$_[0]};
-	}
+    if (!@_) {
+        @_ = {
+            fun => 'function',
+            method => 'method',
+        };
+    }
+    if (@_ == 1 && $_[0] eq ':strict') {
+        @_ = {
+            fun => 'function_strict',
+            method => 'method_strict',
+        };
+    }
+    if (@_ == 1 && ref($_[0]) eq 'HASH') {
+        @_ = map [$_, $_[0]{$_}], keys %{$_[0]};
+    }
 
-	my %spec;
+    my %spec;
 
-	my $bare = 0;
-	for my $proto (@_) {
-		my $item = ref $proto
-			? $proto
-			: [$proto, $bare_arms[$bare++] || confess(qq{Don't know what to do with "$proto"})]
-		;
-		my ($name, $proto_type) = @$item;
-		_assert_valid_identifier $name;
+    my $bare = 0;
+    for my $proto (@_) {
+        my $item = ref $proto
+            ? $proto
+            : [$proto, $bare_arms[$bare++] || confess(qq{Don't know what to do with "$proto"})]
+        ;
+        my ($name, $proto_type) = @$item;
+        _assert_valid_identifier $name;
 
-		unless (ref $proto_type) {
-			# use '||' instead of 'or' to preserve $proto_type in the error message
-			$proto_type = $type_map{$proto_type}
-				|| confess qq["$proto_type" doesn't look like a valid type (one of ${\join ', ', sort keys %type_map})];
-		}
+        $proto_type = {defaults => $proto_type} unless ref $proto_type;
 
-		my %type = %$proto_type;
-		my %clean;
+        my %type = %$proto_type;
+        while (my $defaults = delete $type{defaults}) {
+            my $base = $type_map{$defaults}
+                or confess qq["$defaults" doesn't look like a valid type (one of ${\join ', ', sort keys %type_map})];
+            %type = (%$base, %type);
+        }
 
-		$clean{name} = delete $type{name} || 'optional';
-		$clean{name} =~ /^(?:optional|required|prohibited)\z/
-			or confess qq["$clean{name}" doesn't look like a valid name attribute (one of optional, required, prohibited)];
+        my %clean;
 
-		$clean{shift} = delete $type{shift} || '';
-		_assert_valid_identifier $clean{shift}, 1 if $clean{shift};
+        $clean{name} = delete $type{name} // 'optional';
+        $clean{name} =~ /^(?:optional|required|prohibited)\z/
+            or confess qq["$clean{name}" doesn't look like a valid name attribute (one of optional, required, prohibited)];
 
-		$clean{attrs} = join ' ', map delete $type{$_} || (), qw(attributes attrs);
-		_assert_valid_attributes $clean{attrs} if $clean{attrs};
-		
-		$clean{default_arguments} =
-			exists $type{default_arguments}
-			? !!delete $type{default_arguments}
-			: 1
-		;
-		$clean{check_argument_count} = !!delete $type{check_argument_count};
-		$clean{invocant} = !!delete $type{invocant};
-		$clean{named_parameters} = !!delete $type{named_parameters};
-		$clean{types} = !!delete $type{types};
+        $clean{shift} = delete $type{shift} // '';
+        if ($clean{shift}) {
+            _assert_valid_identifier $clean{shift}, 1;
+            $clean{shift} eq '$_' and confess q[Using "$_" as a parameter is not supported];
+        }
 
-		%type and confess "Invalid keyword property: @{[keys %type]}";
+        $clean{attrs} = join ' ', map delete $type{$_} // (), qw(attributes attrs);
+        _assert_valid_attributes $clean{attrs} if $clean{attrs};
 
-		$spec{$name} = \%clean;
-	}
-	
-	for my $kw (keys %spec) {
-		my $type = $spec{$kw};
+        $clean{default_arguments} = _delete_default \%type, 'default_arguments', 1;
+        $clean{named_parameters}  = _delete_default \%type, 'named_parameters',  1;
+        $clean{types}             = _delete_default \%type, 'types',             1;
 
-		my $flags =
-			$type->{name} eq 'prohibited' ? FLAG_ANON_OK :
-			$type->{name} eq 'required' ? FLAG_NAME_OK :
-			FLAG_ANON_OK | FLAG_NAME_OK
-		;
-		$flags |= FLAG_DEFAULT_ARGS if $type->{default_arguments};
-		$flags |= FLAG_CHECK_NARGS | FLAG_CHECK_TARGS if $type->{check_argument_count};
-		$flags |= FLAG_INVOCANT if $type->{invocant};
-		$flags |= FLAG_NAMED_PARAMS if $type->{named_parameters};
-		$flags |= FLAG_TYPES_OK if $type->{types};
-		$^H{HINTK_FLAGS_ . $kw} = $flags;
-		$^H{HINTK_SHIFT_ . $kw} = $type->{shift};
-		$^H{HINTK_ATTRS_ . $kw} = $type->{attrs};
-		$^H{+HINTK_KEYWORDS} .= "$kw ";
-	}
+        $clean{invocant}             = _delete_default \%type, 'invocant',             0;
+        $clean{runtime}              = _delete_default \%type, 'runtime',              0;
+        $clean{check_argument_count} = _delete_default \%type, 'check_argument_count', 0;
+        $clean{check_argument_types} = _delete_default \%type, 'check_argument_types', 1;
+        $clean{check_argument_count} = $clean{check_argument_types} = 1 if delete $type{strict};
+
+        if (my $rt = delete $type{reify_type}) {
+            ref $rt eq 'CODE' or confess qq{"$rt" doesn't look like a type reifier};
+
+            my $index;
+            for my $i (0 .. $#type_reifiers) {
+                if ($type_reifiers[$i] == $rt) {
+                    $index = $i;
+                    last;
+                }
+            }
+            unless (defined $index) {
+                $index = @type_reifiers;
+                push @type_reifiers, $rt;
+            }
+
+            $clean{reify_type} = $index;
+        }
+
+        %type and confess "Invalid keyword property: @{[keys %type]}";
+
+        $spec{$name} = \%clean;
+    }
+
+    for my $kw (keys %spec) {
+        my $type = $spec{$kw};
+
+        my $flags =
+            $type->{name} eq 'prohibited' ? FLAG_ANON_OK                :
+            $type->{name} eq 'required'   ? FLAG_NAME_OK                :
+                                            FLAG_ANON_OK | FLAG_NAME_OK
+        ;
+        $flags |= FLAG_DEFAULT_ARGS if $type->{default_arguments};
+        $flags |= FLAG_CHECK_NARGS  if $type->{check_argument_count};
+        $flags |= FLAG_CHECK_TARGS  if $type->{check_argument_types};
+        $flags |= FLAG_INVOCANT     if $type->{invocant};
+        $flags |= FLAG_NAMED_PARAMS if $type->{named_parameters};
+        $flags |= FLAG_TYPES_OK     if $type->{types};
+        $flags |= FLAG_RUNTIME      if $type->{runtime};
+        $^H{HINTK_FLAGS_ . $kw} = $flags;
+        $^H{HINTK_SHIFT_ . $kw} = $type->{shift};
+        $^H{HINTK_ATTRS_ . $kw} = $type->{attrs};
+        $^H{HINTK_REIFY_ . $kw} = $type->{reify_type} // 0;
+        $^H{+HINTK_KEYWORDS} .= "$kw ";
+    }
 }
 
 sub unimport {
-	my $class = shift;
+    my $class = shift;
 
-	if (!@_) {
-		delete $^H{+HINTK_KEYWORDS};
-		return;
-	}
+    if (!@_) {
+        delete $^H{+HINTK_KEYWORDS};
+        return;
+    }
 
-	for my $kw (@_) {
-		$^H{+HINTK_KEYWORDS} =~ s/(?<![^ ])\Q$kw\E //g;
-	}
+    for my $kw (@_) {
+        $^H{+HINTK_KEYWORDS} =~ s/(?<![^ ])\Q$kw\E //g;
+    }
 }
 
 
 our %metadata;
 
 sub _register_info {
-	my (
-		$key,
-		$declarator,
-		$invocant,
-		$invocant_type,
-		$positional_required,
-		$positional_optional,
-		$named_required,
-		$named_optional,
-		$slurpy,
-		$slurpy_type,
-	) = @_;
+    my (
+        $key,
+        $declarator,
+        $invocant,
+        $invocant_type,
+        $positional_required,
+        $positional_optional,
+        $named_required,
+        $named_optional,
+        $slurpy,
+        $slurpy_type,
+    ) = @_;
 
-	my $info = {
-		declarator => $declarator,
-		invocant => defined $invocant ? [$invocant, $invocant_type] : undef,
-		slurpy   => defined $slurpy   ? [$slurpy  , $slurpy_type  ] : undef,
-		positional_required => $positional_required,
-		positional_optional => $positional_optional,
-		named_required => $named_required,
-		named_optional => $named_optional,
-	};
+    my $info = {
+        declarator => $declarator,
+        invocant => defined $invocant ? [$invocant, $invocant_type] : undef,
+        slurpy   => defined $slurpy   ? [$slurpy  , $slurpy_type  ] : undef,
+        positional_required => $positional_required,
+        positional_optional => $positional_optional,
+        named_required => $named_required,
+        named_optional => $named_optional,
+    };
 
-	$metadata{$key} = $info;
+    $metadata{$key} = $info;
 }
 
 sub _mkparam1 {
-	my ($pair) = @_;
-	my ($v, $t) = @{$pair || []} or return undef;
-	Function::Parameters::Param->new(
-		name => $v,
-		type => $t,
-	)
+    my ($pair) = @_;
+    my ($v, $t) = @{$pair || []} or return undef;
+    Function::Parameters::Param->new(
+        name => $v,
+        type => $t,
+    )
 }
 
 sub _mkparams {
-	my @r;
-	while (my ($v, $t) = splice @_, 0, 2) {
-		push @r, Function::Parameters::Param->new(
-			name => $v,
-			type => $t,
-		);
-	}
-	\@r
+    my @r;
+    while (my ($v, $t) = splice @_, 0, 2) {
+        push @r, Function::Parameters::Param->new(
+            name => $v,
+            type => $t,
+        );
+    }
+    \@r
 }
 
 sub info {
-	my ($func) = @_;
-	my $key = _cv_root $func or return undef;
-	my $info = $metadata{$key} or return undef;
-	require Function::Parameters::Info;
-	Function::Parameters::Info->new(
-		keyword => $info->{declarator},
-		invocant => _mkparam1($info->{invocant}),
-		slurpy => _mkparam1($info->{slurpy}),
-		(map +("_$_" => _mkparams @{$info->{$_}}), glob '{positional,named}_{required,optional}')
-	)
+    my ($func) = @_;
+    my $key = _cv_root $func or return undef;
+    my $info = $metadata{$key} or return undef;
+    require Function::Parameters::Info;
+    Function::Parameters::Info->new(
+        keyword  => $info->{declarator},
+        invocant => _mkparam1($info->{invocant}),
+        slurpy   => _mkparam1($info->{slurpy}),
+        (map +("_$_" => _mkparams @{$info->{$_}}), glob '{positional,named}_{required,optional}')
+    )
 }
 
 'ok'
@@ -332,20 +387,37 @@ Example:
   #         (1) (6)
     my $f = fun { ... };
 
-In the following section I'm going to describe all parts in order from simplest to most complex.
+In the following section I'm going to describe all parts in order from simplest
+to most complex.
 
 =head3 Body
 
-This is just a normal block of statements, as with L<C<sub>|perlsub>. No surprises here.
+This is just a normal block of statements, as with L<C<sub>|perlsub>. No
+surprises here, except the function name (along with its prototype, if any) is
+already in scope in its body. This is not the case with C<sub>:
+
+  sub foo ($$) {
+  # foo 1, 2;  # a syntax error: 'foo' is not known to be a function yet
+    foo(42);   # OK: calls 'foo' with one argument despite prototype
+  }
+
+On the other hand:
+
+  use Function::Parameters;
+  fun foo :prototype($$) {
+    foo 1, 2;  # OK: 'foo' and its prototype are in scope
+  # foo(42);   # error: Not enough arguments for main::foo at X line Y, near "42)"
+  }
 
 =head3 Name
 
 If present, it specifies the name of the function being defined. As with
-L<C<sub>|perlsub>, if a name is present, the whole declaration is syntactically
-a statement and its effects are performed at compile time (i.e. at runtime you
-can call functions whose definitions only occur later in the file). If no name
-is present, the declaration is an expression that evaluates to a reference to
-the function in question. No surprises here either.
+L<C<sub>|perlsub>, if a name is present, by default the whole declaration is
+syntactically a statement and its effects are performed at compile time (i.e.
+at runtime you can call functions whose definitions only occur later in the
+file - but see the C<runtime> flag below). If no name is present, the
+declaration is an expression that evaluates to a reference to the function in
+question.
 
 =head3 Attributes
 
@@ -366,11 +438,23 @@ L<parameter list|/"Parameter list">. To specify a prototype, put it as the
 first attribute (e.g. C<fun foo :(&$$)>). This is syntactically unambiguous
 because normal L<attributes|/Attributes> need a name after the colon.
 
+You can also use an attribute named C<prototype> (e.g.
+C<fun foo :prototype(&$$)>). In that case it does not have to be the first
+attribute. This syntax is also compatible with C<sub> in perl 5.20 and newer.
+
 =head3 Parameter list
 
 The parameter list is a list of variables enclosed in parentheses, except it's
-actually a bit more complicated than that. A parameter list can include the
-following 6 parts, all of which are optional:
+actually a bit more complicated than that.
+
+Instead of a full variable name (such as C<$foo> or C<@bar>) you can write just
+the sigil (C<$>, C<@>, or C<%>). This has the effect of creating an unnamed
+parameter, which is useful in functions that are called with a certain number
+of arguments but want to ignore one or more of them. This trick works for
+invocants, slurpies, and positional parameters (see below). You can't have
+unnamed named parameters for what I hope are obvious reasons.
+
+A parameter list can include the following 6 parts, all of which are optional:
 
 =over
 
@@ -404,9 +488,9 @@ passed in:
 =item 3. Optional positional parameters
 
 Parameters can be marked as optional by putting an equals sign (C<=>) and an
-expression (the "default argument") after them. If no corresponding argument is
-passed in by the caller, the default argument will be used to initialize the
-parameter:
+(optional) expression (the "default argument") after them. If no corresponding
+argument is passed in by the caller, the default argument will be used to
+initialize the parameter:
 
   fun scale($base, $factor = 2) {
     return $base * $factor;
@@ -415,6 +499,10 @@ parameter:
   say scale(3, 5);  # "15"
   say scale(3);     # "6"
 
+Using just a C<=> with no expression after is equivalent to specifying
+C<= undef>, i.e. the corresponding parameter is optional and has a default
+value of C<undef>.
+
 The default argument is I<not> cached. Every time a function is called with
 some optional arguments missing, the corresponding default arguments are
 evaluated from left to right. This makes no difference for a value like C<2>
@@ -459,7 +547,7 @@ exchange the order of the arguments doesn't matter anymore. As with hash
 initialization, you can specify the same key multiple times and the last
 occurrence wins:
 
-  rectangle(height => 1, width => 2, height => 2, height => 5;
+  rectangle(height => 1, width => 2, height => 2, height => 5);
   # same as: rectangle(width => 2, height => 5);
 
 You can combine positional and named parameters as long as the positional
@@ -474,7 +562,8 @@ parameters come first:
 =item 5. Optional named parameters
 
 As with positional parameters, you can make named parameters optional by
-specifying a default argument after an equals sign (C<=>):
+specifying a default argument (or nothing, which is equivalent to C<undef>)
+after an equals sign (C<=>):
 
   fun rectangle(:$width, :$height, :$color = "chartreuse") {
     ...
@@ -550,12 +639,49 @@ a reference to a hash with the following keys:
 
 =over
 
+=item C<defaults>
+
+Valid values: One of the predefined types C<function>, C<method>,
+C<classmethod>, C<function_strict>, C<method_strict>, C<classmethod_strict>.
+This will set the defaults for all other keys from the specified type, which is
+useful if you only want to override some properties:
+
+  use Function::Parameters { defmethod => { defaults => 'method', shift => '$this' } };
+
+This example defines a keyword called C<defmethod> that works like the standard
+C<method> keyword, but the implicit object variable is called C<$this> instead
+of C<$self>.
+
+Using the string types directly is equivalent to C<defaults> with no further
+customization:
+
+  use Function::Parameters {
+      foo => 'function',         # like: foo => { defaults => 'function' },
+      bar => 'function_strict',  # like: bar => { defaults => 'function_strict' },
+      baz => 'method_strict',    # like: baz => { defaults => 'method_strict' },
+  };
+
 =item C<name>
 
 Valid values: C<optional> (default), C<required> (all functions defined with
 this keyword must have a name), and C<prohibited> (functions defined with this
 keyword must be anonymous).
 
+=item C<runtime>
+
+Valid values: booleans. If enabled, this keyword takes effect at runtime, not
+compile time:
+
+  use Function::Parameters { fun => { defaults => 'function_strict', runtime => 1 } };
+  say defined &foo ? "defined" : "not defined";  # not defined
+  fun foo() {}
+  say defined &foo ? "defined" : "not defined";  # defined
+
+C<&foo> is only defined after C<fun foo() {}> has been reached at runtime.
+
+B<CAVEAT:> A future version of this module may enable C<< runtime => 1 >> by
+default for methods.
+
 =item C<shift>
 
 Valid values: strings that look like scalar variables. This lets you specify a
@@ -590,15 +716,43 @@ automatically check that they have been passed all required arguments and no
 excess arguments. If this check fails, an exception will by thrown via
 L<C<Carp::croak>|Carp>.
 
+=item C<check_argument_types>
+
+Valid values: booleans. This property is on by default; use
+C<< check_argument_types => 0 >> to turn it off. This controls whether
+functions defined with this keyword automatically check that the arguments they
+are passed pass the declared type constraints (if any). See
+L</Experimental feature: Types> below.
+
+=item C<strict>
+
+Valid values: booleans. This turns on both C<check_argument_count> and
+C<check_argument_types>.
+
+=item C<reify_type>
+
+Valid values: code references. The function specified here will be called to
+turn type annotations into constraint objects (see
+L</Experimental feature: Types> below). It will receive two arguments: a string
+containing the type description, and the name of the current package.
+
+The default type reifier is equivalent to:
+
+ sub {
+     require Moose::Util::TypeConstraints;
+     Moose::Util::TypeConstraints::find_or_create_isa_type_constraint($_[0])
+ }
+
 =back
 
 The predefined type C<function> is equivalent to:
 
  {
-   name => 'optional',
-   invocant => 0,
+   name              => 'optional',
    default_arguments => 1,
-   check_argument_count => 0,
+   strict            => 0,
+   invocant          => 0,
+   runtime           => 0,
  }
 
 These are all default values, so C<function> is also equivalent to C<{}>.
@@ -606,29 +760,24 @@ These are all default values, so C<function> is also equivalent to C<{}>.
 C<method> is equivalent to:
 
  {
-   name => 'optional',
-   shift => '$self',
-   invocant => 1,
-   attributes => ':method',
-   default_arguments => 1,
-   check_argument_count => 0,
+   defaults          => 'function',
+   attributes        => ':method',
+   shift             => '$self',
+   invocant          => 1,
+   # runtime         => 1,  ## possibly in a future version of this module
  }
 
 
 C<classmethod> is equivalent to:
 
  {
-   name => 'optional',
-   shift => '$class',
-   invocant => 1,
-   attributes => ':method',
-   default_arguments => 1,
-   check_argument_count => 0,
+   defaults          => 'method',
+   shift             => '$class',
  }
 
 C<function_strict>, C<method_strict>, and
 C<classmethod_strict> are like C<function>, C<method>, and
-C<classmethod>, respectively, but with C<< check_argument_count => 1 >>.
+C<classmethod>, respectively, but with C<< strict => 1 >>.
 
 =back
 
@@ -669,15 +818,17 @@ affects the file that is currently being compiled.
 =head2 Experimental feature: Types
 
 An experimental feature is now available: You can annotate parameters with
-L<Moose types|Moose::Manual::Types>. That is, before each parameter you can put
-a type specification consisting of identifiers (C<Foo>), unions (C<... | ...>),
-and parametric types (C<...[...]>). Example:
+types. That is, before each parameter you can put a type specification
+consisting of identifiers (C<Foo>), unions (C<... | ...>), and parametric types
+(C<...[...]>). Example:
 
-  fun foo(Int $n, ArrayRef[String | CodeRef] $cb) { ... }
+  fun foo(Int $n, ArrayRef[Str | CodeRef] $cb) { ... }
 
-If you do this, L<Moose> will be loaded automatically (if that hasn't happened
-yet). These specifications are parsed and validated using
-L<C<Moose::Util::TypeConstraints::find_or_parse_type_constraint>|Moose::Util::TypeConstraints/find_or_parse_type_constraint>.
+If you do this, the type reification function corresponding to the keyword will
+be called to turn the type (a string) into a constraint object. The default
+type reifier simply loads L<Moose> and forwards to
+L<C<Moose::Util::TypeConstraints::find_or_parse_type_constraint>|Moose::Util::TypeConstraints/find_or_parse_type_constraint>,
+which creates L<Moose types|Moose::Manual::Types>.
 
 If you are in "lax" mode, nothing further happens and the types are ignored. If
 you are in "strict" mode, C<Function::Parameters> generates code to make sure
@@ -721,6 +872,12 @@ generated code corresponds to:
   # ... turns into ...
   sub bar :method { my $self = shift; my ($x, $y, @z) = @_; sub bar; ... }
 
+=head1 BUGS AND INCOMPATIBILITIES
+
+A future version of this module may enable C<< runtime => 1 >> by default for
+methods. If this would break your code, please send me a note or file a bug on
+RT.
+
 =head1 SUPPORT AND DOCUMENTATION
 
 After installing, you can find documentation for this module with the
@@ -764,7 +921,7 @@ Lukas Mai, C<< <l.mai at web.de> >>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2010-2013 Lukas Mai.
+Copyright 2010-2014 Lukas Mai.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
